#----------------------------------------------------------
#File: LED_Class.py
#Environment: Raspberry Pi 3B/Python 3.10
#Purpose: Class to control the built-in LEDs on the Raspberry Pie 3B/1.2
#Author: GravelSocks
#Date: 2021.10.18
#----------------------------------------------------------

class RPi_LED:

#----------------------------------------------------------
#constructor
    def __init__(self,led_name):
        match (led_name):
            case 'sys_red':
                self.name = 'led1'
            case 'sys_green':
                self.name = 'led0'
        self.location = '/sys/class/leds/'
        self.control = self.location+self.name+'/brightness'
        self.on = '1'
        self.off = '0'
        self.state = 0 
        print("created RPi_LED class for: ",self.name)
        
#----------------------------------------------------------
#destructor
    def __del__(self):
        print("Deleted RPi_LED class for: ",self.name)

#----------------------------------------------------------
    def get_led(self):
        """
        Function: get_led: get the LED state
        Description: check if the led for this class is on or off
        Parameters: N/A
        Returns: 1 if on, 0 if off, or error value
        """
 
        cmd = 'cat '+self.control
        import subprocess
        result = subprocess.check_output(cmd,shell=True).decode('utf-8').strip()
# Alternate parsing method, but subprocess may be preferable
#       import os
#       result = os.popen(cmd).read().strip()
        match (result):
            case '255':
                return 1
            case '0':
                return 0
            case _:
                print("Error in get_led - unexpected value: ",result)
                return result

#class methods
#----------------------------------------------------------
    def set_led(self,state):
        """
        Function: set_led: set the LED state
        Description: turn the LED on or off
        Parameters: state(int): state to set. 0: off; 1: on
        Returns: N/A
        """
        import os
#        import sys
        match (state):
            case 0:
                level = self.off
                self.state = state
            case 1:
                level = self.on
                self.state = state
            case _:
                print("bad brightness level")
#flush any pending output - needed?
#        sys.stdout.flush()
#write the command
        os.system('echo '+level+' > '+self.control)
#flush our output just in case - value is out of sync - needed?
#        sys.stdout.flush()

#----------------------------------------------------------
    def led_on(self,debug=False):
        """
        Function: led_on: turn on the led for this class
        Description: macro method to call set_led to turn on led
        Parameters: debug: debug flag to print activity
        Returns: N/A
        """
        if (debug):
            print("turning on: ",self.name)
        self.set_led(1)

#----------------------------------------------------------
    def led_off(self,debug=False):
        """
        Function: led_off: turn off the led for this class
        Description: macro method to call set_led to turn off led
        Parameters: debug: debug flag to print activity
        Returns: N/A
        """
        if (debug):
            print("turning off: ",self.name)
        self.set_led(0)

#----------------------------------------------------------
    def check_led(self,debug=False):
        """
        Function: check_led: check led status
        Description: compare the brightness value with the last known state
        Parameters: debug: debug flag to print status - default is False
        Returns: the value stored in brightness
        """
        result = self.get_led()
        if (debug):
            if (result != self.state):
                print ("ERROR: ",self.name," STATE IS INCORRECT")
            else:
                print (self.name," state matches last write")
        return result
    
#----------------------------------------------------------

def test_leds(args=None):
    #create the objects
    green = RPi_LED('sys_green')
    red = RPi_LED('sys_red')

    #turn on both system LEDs
    green.led_on(True)
    red.led_on(True)

    #check the LED status
    print("green on: ",green.check_led(True),flush=True)
    print("red on: ",red.check_led(True),flush=True)

    #turn off both system LEDs
    green.led_off(True)
    red.led_off(True)

    #check the LED status
    print("green off: ",green.check_led(True),flush=True)
    print("red off: ",red.check_led(True),flush=True)

    #delete the objects
    del green
    del red

def main(args=None):
    test_leds()
    
if __name__ == '__main__':
    main()
